#extension GL_OES_EGL_image_external : require
precision highp float;
varying vec2 textureCoordinate;
uniform samplerExternalOES vTexture;

uniform float displayMaxLuminance ;

highp vec3 ToneMap(highp vec3 color) {
    float maxMasteringLumi = 1000.0;
    float maxContentLumi = 1000.0;
    float maxInLumi = min(maxMasteringLumi, maxContentLumi);
    float maxOutLumi = displayMaxLuminance;

    float nits = color.y;

    // clamp to max input luminance
    nits = clamp(nits, 0.0, maxInLumi);

    // scale [0.0, maxInLumi] to [0.0, maxOutLumi]
    if (maxInLumi <= maxOutLumi) {
        return color * (maxOutLumi / maxInLumi);
    } else {
        // three control points
        const float x0 = 10.0;
        const float y0 = 17.0;
        float x1 = maxOutLumi * 0.75;
        float y1 = x1;
        float x2 = x1 + (maxInLumi - x1) / 2.0;
        float y2 = y1 + (maxOutLumi - y1) * 0.75;

        // horizontal distances between the last three control points
        float h12 = x2 - x1;
        float h23 = maxInLumi - x2;
        // tangents at the last three control points
        float m1 = (y2 - y1) / h12;
        float m3 = (maxOutLumi - y2) / h23;
        float m2 = (m1 + m3) / 2.0;

        if (nits < x0) {
            // scale [0.0, x0] to [0.0, y0] linearly
            float slope = y0 / x0;
            return color * slope;
        } else if (nits < x1) {
            // scale [x0, x1] to [y0, y1] linearly
            float slope = (y1 - y0) / (x1 - x0);
            nits = y0 + (nits - x0) * slope;
        } else if (nits < x2) {
            // scale [x1, x2] to [y1, y2] using Hermite interp
            float t = (nits - x1) / h12;
            nits = (y1 * (1.0 + 2.0 * t) + h12 * m1 * t) * (1.0 - t) * (1.0 - t) +
            (y2 * (3.0 - 2.0 * t) + h12 * m2 * (t - 1.0)) * t * t;
        } else {
            // scale [x2, maxInLumi] to [y2, maxOutLumi] using Hermite interp
            float t = (nits - x2) / h23;
            nits = (y2 * (1.0 + 2.0 * t) + h23 * m2 * t) * (1.0 - t) * (1.0 - t) +
            (maxOutLumi * (3.0 - 2.0 * t) + h23 * m3 * (t - 1.0)) * t * t;
        }
    }

    // color.y is greater than x0 and is thus non-zero
    return color * (nits / color.y);
}

highp float EOTF_channel(const highp float channel) {
    const highp float a = 0.17883277;
    const highp float b = 0.28466892;
    const highp float c = 0.55991073;
    return channel <= 0.5 ? channel * channel / 3.0 :
    (exp((channel - c) / a) + b) / 12.0;
}

vec3 EOTF(const highp vec3 color) {
    return vec3(EOTF_channel(color.r), EOTF_channel(color.g),
                EOTF_channel(color.b));
}

highp vec3 ScaleLuminance(highp vec3 color) {
    // The formula is:
    // alpha * pow(Y, gamma - 1.0) * color + beta;
    // where alpha is displayMaxLuminance, gamma is 1.2, beta is 0.0.
    return color * 1000.0 * pow(color.y, 0.2);
}

//highp vec3 NormalizeLuminance(highp vec3 color) {
//    return color / 1000.0 * pow(color.y / 1000.0, -0.2 / 1.2);
//}
highp vec3 NormalizeLuminance(highp vec3 color) {
    return color / displayMaxLuminance;
}

highp vec3 OOTF(const highp vec3 color) {
    return NormalizeLuminance(ToneMap(ScaleLuminance(color)));
}

//
//float OETF_BT709_f(const float linear) {
//    return linear < 0.018 ? linear * 4.5 : (pow(linear, 1.0 / 2.2) * 1.099) - 0.099;
//}
//
//vec3 OETF_BT709(const vec3 linear) {
//    return vec3(OETF_BT709_f(linear.r), OETF_BT709_f(linear.g), OETF_BT709_f(linear.b));
//}
//
//vec3 OETF(const vec3 linear) {
//    return sign(linear.rgb) * OETF_BT709(abs(linear.rgb));
//}

float OETF_sRGB_f(const float linear) {
    return linear <= 0.0031308 ?
    linear * 12.92 : (pow(linear, 1.0 / 2.4) * 1.055) - 0.055;
}

vec3 OETF_sRGB(const vec3 linear) {
    return vec3(OETF_sRGB_f(linear.r), OETF_sRGB_f(linear.g), OETF_sRGB_f(linear.b));
}

vec3 OETF(const vec3 linear) {
    return sign(linear.rgb) * OETF_sRGB(abs(linear.rgb));
}



highp vec3 InputTransform(const highp vec3 color) {

    //    0.6370, 0.1446, 0.1689,
    //    0.2627, 0.6780, 0.0593,
    //    0.0000, 0.0281, 1.0610

    const mat3 bt2020RGBtoXYZ = mat3(
        0.6370, 0.2627, 0.0000,
        0.1446, 0.6780, 0.0281,
        0.1689, 0.0593, 1.0610);
    return clamp(vec3(bt2020RGBtoXYZ * color), 0.0, 1.0);
    //    return color;
}


highp vec3 OutputTransform(const highp vec3 color) {

    //    const mat3 bt2020ToBT709 = mat3(
    //        1.660491, -0.12455047, -0.01815076,
    //        -0.58764114, 1.1328999, -0.1005789,
    //        -0.07284986, -0.00834942, 1.11872966);

    //    3.2410, -1.5374, -0.4986,
    //    -0.9692, 1.8760, 0.0416,
    //    0.0556, -0.2040, 1.0570
    const mat3 bt709XYZTORGB = mat3(
        3.2410, -0.9692, 0.0556,
        -1.5374, 1.8760, -0.2040,
        -0.4986, 0.0416, 1.0570);

    //    1.7167, -0.3557, -0.2534
    //    -0.6667, 1.6165, 0.0158
    //    0.0176, -0.0428, 0.9421
    //const mat3 bt2020XYZTORGB = mat3(
    //    1.7167, -0.6667, 0.0176,
    //    -0.3557, 1.6165, -0.0428,
    //    -0.2534, 0.0158, 0.9421);

    //    return clamp(vec3(bt2020ToBT709 * color  ), 0.0, 1.0);
    return clamp(vec3(bt709XYZTORGB * color), 0.0, 1.0);
}


void main()
{
    vec4 hdr = texture2D(vTexture, textureCoordinate);
    vec3 color = hdr.rgb;
    color = EOTF(color);
    color = InputTransform(color);
    color = OOTF(color);
    //
    color = OutputTransform(color);
    color = OETF(color);
    //    vec3 color = OETF(OutputTransform(OOTF(InputTransform(EOTF(hdr.rgb)))));
    gl_FragColor = vec4(color, hdr.a);
    //    gl_FragColor = hdr;
}
