#extension GL_OES_EGL_image_external : require
precision highp float;
varying vec2 textureCoordinate;

uniform samplerExternalOES vTexture;

uniform float displayMaxLuminance;
uniform float maxMasteringLuminance;
uniform float maxContentLuminance;

highp vec3 ToneMap(highp vec3 color) {
    float maxMasteringLumi = maxMasteringLuminance;
    float maxContentLumi = maxContentLuminance;
    float maxInLumi = min(maxMasteringLumi, maxContentLumi);
    float maxOutLumi = displayMaxLuminance;

    float nits = color.y;

    // clamp to max input luminance
    nits = clamp(nits, 0.0, maxInLumi);

    // scale [0.0, maxInLumi] to [0.0, maxOutLumi]
    if (maxInLumi <= maxOutLumi) {
        return color * (maxOutLumi / maxInLumi);
    } else {
        // three control points
        const float x0 = 10.0;
        const float y0 = 17.0;
        float x1 = maxOutLumi * 0.75;
        float y1 = x1;
        float x2 = x1 + (maxInLumi - x1) / 2.0;
        float y2 = y1 + (maxOutLumi - y1) * 0.75;

        // horizontal distances between the last three control points
        float h12 = x2 - x1;
        float h23 = maxInLumi - x2;
        // tangents at the last three control points
        float m1 = (y2 - y1) / h12;
        float m3 = (maxOutLumi - y2) / h23;
        float m2 = (m1 + m3) / 2.0;

        if (nits < x0) {
            // scale [0.0, x0] to [0.0, y0] linearly
            float slope = y0 / x0;
            return color * slope;
        } else if (nits < x1) {
            // scale [x0, x1] to [y0, y1] linearly
            float slope = (y1 - y0) / (x1 - x0);
            nits = y0 + (nits - x0) * slope;
        } else if (nits < x2) {
            // scale [x1, x2] to [y1, y2] using Hermite interp
            float t = (nits - x1) / h12;
            nits = (y1 * (1.0 + 2.0 * t) + h12 * m1 * t) * (1.0 - t) * (1.0 - t) +
            (y2 * (3.0 - 2.0 * t) + h12 * m2 * (t - 1.0)) * t * t;
        } else {
            // scale [x2, maxInLumi] to [y2, maxOutLumi] using Hermite interp
            float t = (nits - x2) / h23;
            nits = (y2 * (1.0 + 2.0 * t) + h23 * m2 * t) * (1.0 - t) * (1.0 - t) +
            (maxOutLumi * (3.0 - 2.0 * t) + h23 * m3 * (t - 1.0)) * t * t;
        }
    }

    // color.y is greater than x0 and is thus non-zero
    return color * (nits / color.y);
}


float OETF_BT709_f(const float linear) {
    return linear <= 0.018 ? linear * 4.5 : (pow(linear, 1.0 / 2.2) * 1.099) - 0.099;
}

vec3 OETF_BT709(const vec3 linear) {
    return vec3(OETF_BT709_f(linear.r), OETF_BT709_f(linear.g), OETF_BT709_f(linear.b));
}

vec3 OETF(const vec3 linear) {
    return sign(linear.rgb) * OETF_BT709(abs(linear.rgb));
}


vec3 EOTF(const highp vec3 color) {
    const highp float m1 = (2610.0 / 4096.0) / 4.0;
    const highp float m2 = (2523.0 / 4096.0) * 128.0;
    const highp float c1 = (3424.0 / 4096.0);
    const highp float c2 = (2413.0 / 4096.0) * 32.0;
    const highp float c3 = (2392.0 / 4096.0) * 32.0;

    highp vec3 tmp = pow(clamp(color, 0.0, 1.0), 1.0 / vec3(m2));
    tmp = max(tmp - c1, 0.0) / (c2 - c3 * tmp);
    return pow(tmp, 1.0 / vec3(m1));
}

highp vec3 InputTransform(const highp vec3 color) {
    return color;
}

highp vec3 ScaleLuminance(highp vec3 color) {
    return color * 10000.0;
}

highp vec3 NormalizeLuminance(highp vec3 color) {
    return color / 10000.0;
}

highp vec3 OOTF(const highp vec3 color) {
    return NormalizeLuminance(ToneMap(ScaleLuminance(color)));
}

highp vec3 OutputTransform(const highp vec3 color) {
    const mat3 outputTransformMatrix = mat3(
        1.660491,   -0.12455047, -0.01815076,
        -0.58764114,  1.1328999,  -0.1005789,
        -0.07284986, -0.00834942,  1.11872966);
    return clamp(vec3(outputTransformMatrix * color), 0.0, 1.0);
}

void main()
{
    vec4 hdr = texture2D(vTexture, textureCoordinate);
    vec3 color = EOTF(hdr.rgb);
    color = InputTransform(color);
    color = OOTF(color);
    color = OutputTransform(color);
    color = OETF(color);
    //    OETF(OutputTransform(OOTF(InputTransform(EOTF(hdr.rgb)))));
    gl_FragColor = vec4(color, hdr.a);
//        gl_FragColor = hdr;
}
